searchData={"items":[{"type":"module","title":"Liberator.Conn","doc":"Utility functions for dealing with `Plug.Conn` structs.","ref":"Liberator.Conn.html"},{"type":"function","title":"Liberator.Conn.read_body/2","doc":"Reads the body from the conn, and puts it in the assigns under the key `:raw_body`.\n\nAccepts the same headers as `Plug.Conn.read_body/2`:\n\n* `:length` - sets the maximum number of bytes to read from the body for\n  each chunk, defaults to `64_000` bytes\n* `:read_length` - sets the amount of bytes to read at one time from the\n  underlying socket to fill the chunk, defaults to `64_000` bytes\n* `:read_timeout` - sets the timeout for each socket read, defaults to\n  `5_000` milliseconds","ref":"Liberator.Conn.html#read_body/2"},{"type":"behaviour","title":"Liberator.Encoding","doc":"A behaviour module for compression codecs.\n\nLiberator uses this behaviour to help make sure at compile-time that codecs will be called successfully.\nInclude it in your own module for the same peace of mind.","ref":"Liberator.Encoding.html"},{"type":"callback","title":"Liberator.Encoding.encode!/1","doc":"Encode a binary into an encoded form, and raises if there's an error.","ref":"Liberator.Encoding.html#c:encode!/1"},{"type":"behaviour","title":"Liberator.MediaType","doc":"A behaviour module for media type codecs.\n\nLiberator uses this behaviour to help make sure at compile-time that codecs will be called successfully.\nInclude it in your own module for the same peace of mind.","ref":"Liberator.MediaType.html"},{"type":"callback","title":"Liberator.MediaType.encode!/1","doc":"Encode an Elixir term into an encoded binary, and raise if there's an error.","ref":"Liberator.MediaType.html#c:encode!/1"},{"type":"behaviour","title":"Liberator.Resource","doc":"A controller module that understands and respects the HTTP spec.\n\nThis module implements a `Plug` handler that allows an endpoint to comply to the HTTP specification,\nand to do so by just answering a few questions.\n\nDefine a simple resource like this:\n\n    defmodule MyFirstResource do\n      use Liberator.Resource\n\n      def handle_ok(_), do: \"Hello world!\"\n    end\n\nTo add this plug to a Phoenix application, use the `Phoenix.Router.forward/4` keyword in your router:\n\n    scope \"/\", MyApp do\n      pipe_through [:browser]\n\n      forward \"/api/resource\", MyFirstResource\n    end\n\nIf you're using another Plug-based framework, use `Plug.forward/4` once you've matched on the path:\n\n    defmodule Router do\n     def init(opts), do: opts\n\n     def call(conn, opts) do\n       case conn do\n         %{host: \"localhost\", path_info: [\"resources\" | rest]} ->\n           Plug.forward(conn, rest, MyFirstResource, opts)\n\n         _ ->\n           MainRouter.call(conn, opts)\n       end\n     end\n    end\n\nThere are lots of decisions to be made during content negotiation,\nand Liberator lets gives you access to every single one,\nbut it's also built with sensible defaults that let you quickly build up a controller.","ref":"Liberator.Resource.html"},{"type":"behaviour","title":"Content Negotiation - Liberator.Resource","doc":"These functions return lists of available values during content negotiation.\nIf the client has an accept header that does not match a value from these lists,\nthe plug will return a `406 Not Acceptable` response, and call `c:handle_not_acceptable/1`.\n\n| Function                    | Default                             |\n|-----------------------------|-------------------------------------|\n| `c:allowed_methods/1`       | `[\"GET\", \"HEAD\"]`                   |\n| `c:known_methods/1`         | `[\"GET\", \"HEAD\", \"OPTIONS\", \"PUT\", \"POST\", \"DELETE\", \"PATCH\", \"TRACE\"]` |\n| `c:available_media_types/1` | `[\"text/plain\", \"application/json\"]`|\n| `c:available_languages/1`   | `[\"*\"]`                             |\n| `c:available_charsets/1`    | `[\"UTF-8\"]`                         |\n| `c:available_encodings/1`   | `[\"gzip\", \"deflate\",\"identity\"]`    |\n| `c:maximum_entity_length/1` | `64_000`                            |\n\nLiberator supports a few basic defaults to help you get up and running.\nIt uses `Jason` for `application/json` media responses,\nbut you can override that, along with the compression handlers.\nJust include your custom codecs in `config.exs` under the `:liberator` key.\nMedia type codecs go under `:media_types`, and compression goes under `:encodings`.\nHere's the default as an example:\n\n    config :liberator,\n      media_types: %{\n        \"text/plain\" => Liberator.MediaType.TextPlain,\n        \"application/json\" => Jason\n      },\n      encodings: %{\n        \"identity\" => Liberator.Encoding.Identity,\n        \"deflate\" => Liberator.Encoding.Deflate,\n        \"gzip\" => Liberator.Encoding.Gzip\n      }\n\nAs long as your codec module implements an `encode!/1` function that accepts and returns a response body,\nLiberator will call it at the right place in the pipeline.\nImplement the `Liberator.MediaType` or `Liberator.Encoding` behaviour for some compile-time assurance that you've implemented the correct function.\n\n    defmodule MyXmlCodec do\n      @behaviour Liberator.MediaType\n\n      @impl true\n      def encode!(body) do\n        # your cool new functionality here\n      end\n    end","ref":"Liberator.Resource.html#module-content-negotiation"},{"type":"behaviour","title":"Preconditions - Liberator.Resource","doc":"These functions decide the state of preconditon decisions for the request.\nDepending on the specific method and request headers,\nthe plug may return a `412 Precondition Failed` response, a `304 Not Modified` response,\nor may allow another kind of request to continue.\n\n| Function                    | Default              |\n|-----------------------------|----------------------|\n| `c:last_modified/1`         | `DateTime.utc_now()` |\n| `c:etag/1`                  | `nil`                |","ref":"Liberator.Resource.html#module-preconditions"},{"type":"behaviour","title":"Actions - Liberator.Resource","doc":"Actions make the necessary changes to the requested entity.\nYou can return either a `Plug.Conn` struct, or a map, from these functions.\nHowever, unlike the decision functions, a nil or false return value does nothing different.\n\n| Function          | Description |\n|-------------------|-------------|\n| `c:initialize/1` | Performs any custom initialization you need before the decision tree starts |\n| `c:delete!/1`    | Called for `DELETE` requests |\n| `c:patch!/1`     | Called for `PATCH` requests  |\n| `c:post!/1`      | Called for `POST` requests   |\n| `c:put!/1`       | Called for `PUT` requests    |","ref":"Liberator.Resource.html#module-actions"},{"type":"behaviour","title":"Handlers - Liberator.Resource","doc":"Handlers are called at the very end of the decision tree, and allow you\nto return content for rendering to the client.\n\n| Function                                    | Status|\n|---------------------------------------------|-------|\n| `c:handle_ok/1`                             | 200 |\n| `c:handle_options/1`                        | 200 |\n| `c:handle_created/1`                        | 201 |\n| `c:handle_accepted/1`                       | 202 |\n| `c:handle_no_content/1`                     | 204 |\n| `c:handle_multiple_representations/1`       | 300 |\n| `c:handle_moved_permanently/1`              | 301 |\n| `c:handle_see_other/1`                      | 303 |\n| `c:handle_not_modified/1`                   | 304 |\n| `c:handle_moved_temporarily/1`              | 307 |\n| `c:handle_malformed/1`                      | 400 |\n| `c:handle_unauthorized/1`                   | 401 |\n| `c:handle_forbidden/1`                      | 403 |\n| `c:handle_not_found/1`                      | 404 |\n| `c:handle_method_not_allowed/1`             | 405 |\n| `c:handle_not_acceptable/1`                 | 406 |\n| `c:handle_conflict/1`                       | 409 |\n| `c:handle_gone/1`                           | 410 |\n| `c:handle_precondition_failed/1`            | 412 |\n| `c:handle_request_entity_too_large/1`       | 413 |\n| `c:handle_uri_too_long/1`                   | 414 |\n| `c:handle_unsupported_media_type/1`         | 415 |\n| `c:handle_unprocessable_entity/1`           | 422 |\n| `c:handle_too_many_requests/1`              | 429 |\n| `c:handle_unavailable_for_legal_reasons/1`  | 451 |\n| `c:handle_unknown_method/1`                 | 501 |\n| `c:handle_not_implemented/1`                | 501 |\n| `c:handle_service_unavailable/1`            | 503 |","ref":"Liberator.Resource.html#module-handlers"},{"type":"behaviour","title":"Decisions - Liberator.Resource","doc":"Liberator supports a whole lot of decisions points.\nSome of them are needed for next to every resource definition.\nOthers are seldom used or there is no other sensible implementation.\n\nDecision callbacks must return a truthy value, which they can optionally wrap in an `{:ok, result}` tuple.\nReturning `{:error, result}` will always invoke the `handle_error`; see below.\n\nIf the result of the decision is a map, Liberator will merge that map with the `conn`'s `:assigns` map.\nUse this feature to cache data and do work when it makes sense.\nFor example, the `exists?/1` callback is a great place to fetch your resource,\nand you can return it as a map for your later functions to act upon.\nThat would look something like this:\n\n    def exists?(conn) do\n      id = List.last(conn.path_info)\n      try do\n        Hello.Blog.get_post!(id)\n      rescue\n        Ecto.NoResultsError -> false\n        ArgumentError -> false\n      else\n        post -> %{post: post}\n      end\n    end\n\nHere are all the decision functions you can override:\n\n| Function                    | Description                                                                         | Default  |\n|---                          |---                                                                                  |---       |\n| `c:allowed?/1`              | Is the user allowed to make this request?                                           | true     |\n| `c:authorized?/1`           | Is necessary authentication information present?                                    | true     |\n| `c:charset_available?/1`    | Are any of the requested charsets available? Should assign the `:charset` variable. | Uses values at `c:available_charsets/1` |\n| `c:can_post_to_gone?/1`     | Should we process a `POST` to a resource that previously existed?                   | false |\n| `c:can_post_to_missing?/1`  | Should we process a `POST` to a resource that does not exist?                       | true |\n| `c:can_put_to_missing?/1`   | Should we process a `PUT` to a resource that does not exist?                        | true |\n| `c:conflict?/1`             | Does the `PUT` or `POST` request result in a conflict?                              | false |\n| `c:delete_enacted?/1`       | Was the delete request finally processed?                                           | true |\n| `c:encoding_available?/1`   | Is the requested encoding available? Should assign the `:encoding` variable.        | Uses values at `c:available_encodings/1` |\n| `c:etag_matches_for_if_match?/1` | Does the etag of the current resource match the `If-Match` header?             | Uses value generated by `c:etag/1` |\n| `c:etag_matches_for_if_none?/1` | Does the etag of the current resource match the `If-None-Match` header?         | Uses value generated by `c:etag/1` |\n| `c:existed?/1`              | Did the resource exist before?                                                      | false |\n| `c:exists?/1`               | Does the resource exist?                                                            | true |\n| `c:known_content_type?/1`   | Is the `Content-Type` of the body known?                                            | true |\n| `c:known_method?/1`         | Is the request method known?                                                        | Uses values at `c:known_methods/1` |\n| `c:language_available?/1`   | Is the requested language available? Should assign the `:language` variable.        | Uses values at `c:available_languages/1` |\n| `c:media_type_available?/1` | Is the requested media type available? Should assign the `:media_type` variale.     | Uses values at `c:available_media_types/1` |\n| `c:method_allowed?/1`       | Is the request method allowed for this resource?                                    | Uses values at `c:allowed_methods/1` |\n| `c:modified_since?/1`       | Was the resource modified since the date given in the `If-Modified-Since` header?   | Uses value generated by `c:last_modified/1` |\n| `c:moved_permanently?/1`    | Was the resource moved permanently?                                                 | false |\n| `c:moved_temporarily?/1`    | Was the resource moved temporarily?                                                 | false |\n| `c:multiple_representations?/1` | Are there multiple representations for this resource?                           | false |\n| `c:new?/1`                  | Was the resource created by this request?                                           | true |\n| `c:payment_required?/1`     | Is payment required before this request can be processed?                           | false |\n| `c:post_enacted?/1`         | Was the `POST` request finally processed?                                           | true |\n| `c:put_enacted?/1`          | Was the `PUT` request finally processed?                                            | true |\n| `c:patch_enacted?/1`        | Was the `PATCH` request finally processed?                                          | true |\n| `c:post_redirect?/1`        | Should the response redirect after a `POST`?                                        | false |\n| `c:put_to_different_url?/1` | Should the `PUT` request be made to a different URL?                                | false |\n| `c:processable?/1`          | Is the request body processable?                                                    | true |\n| `c:too_many_requests?/1`    | Has the client or user issued too many requests in a period of time?                | false |\n| `c:service_available?/1`    | Is the service available?                                                           | true |\n| `c:unavailable_for_legal_reasons?/1` | Is the resource not available, for legal reasons?                          | false |\n| `c:uri_too_long?/1`         | Is the request URI too long?                                                        | false |\n| `c:valid_content_header?/1` | Is the `Content-Type` of the body valid?                                            | true |\n| `c:valid_entity_length?/1`  | Is the length of the body valid?                                                    | Uses value at `c:maximum_entity_length/1` |\n| `c:well_formed?/1`            | Is the request parseable?                                                         | true |","ref":"Liberator.Resource.html#module-decisions"},{"type":"behaviour","title":"Handling Errors - Liberator.Resource","doc":"There is a special handler, named `c:handle_error/3`, that is called when any decision, action, or handler function raises an error or returns `{:error, result}`.\nIt functions much like an [`action_fallback`](https://hexdocs.pm/phoenix/controllers.html#action-fallback) module does in Phoenix.\n\nThe `handle_error` handler is called with the `conn`, the error that was raised, and the name of the decision, action, or handler that failed.\nLiberator expects this handler to call `Plug.Conn.send_resp/1` or `Plug.Conn.send_resp/3`, unlike other handlers.\nThis allows you to set the status and body yourself, or even use a Phoenix fallback controller.\n\nThe default implementation of `handle_error` works something like this:\n\n    @impl true\n    def handle_error(conn, _error, _failed_step) do\n      send_resp(conn, 500, \"Internal Server Error\")\n    end","ref":"Liberator.Resource.html#module-handling-errors"},{"type":"behaviour","title":"Debugging - Liberator.Resource","doc":"For every request, Liberator builds a list of the decisions called and their answers.\nYou can access this list with the `:trace` option of your `use` statement.\n\nSet it to `:log` for a log message of the full trace.\n\n    defmodule MyFirstResource do\n      use Liberator.Resource, trace: :log\n\n      def handle_ok(_), do: \"Hello world!\"\n    end\n\nYou'll get a log message like this:\n\n```comment\n14:57:04.861 [debug] Liberator trace for request \"my-very-specific-request-id\" to /:\n\n    1. initialize: nil (took 1 µs)\n    2. service_available?: true (took 1 µs)\n    3. known_method?: true (took 1 µs)\n    4. uri_too_long?: false (took 1 µs)\n    5. method_allowed?: true (took 1 µs)\n    6. well_formed?: true (took 1 µs)\n    7. authorized?: true (took 1 µs)\n    8. allowed?: true (took 1 µs)\n    9. too_many_requests?: false (took 1 µs)\n    10. payment_required?: false (took 1 µs)\n    11. valid_content_header?: true (took 1 µs)\n    12. known_content_type?: true (took 1 µs)\n    13. valid_entity_length?: true (took 1 µs)\n    14. method_options?: false (took 1 µs)\n    15. accept_exists?: false (took 1 µs)\n    16. accept_language_exists?: false (took 1 µs)\n    17. accept_charset_exists?: false (took 1 µs)\n    18. accept_encoding_exists?: false (took 1 µs)\n    19. processable?: true (took 1 µs)\n    20. unavailable_for_legal_reasons?: false (took 1 µs)\n    21. exists?: true (took 1 µs)\n    22. if_match_exists?: false (took 1 µs)\n    23. if_unmodified_since_exists?: false (took 1 µs)\n    24. if_none_match_exists?: false (took 1 µs)\n    25. if_modified_since_exists?: false (took 1 µs)\n    26. method_delete?: false (took 1 µs)\n    27. method_patch?: false (took 1 µs)\n    28. post_to_existing?: false (took 1 µs)\n    29. put_to_existing?: false (took 1 µs)\n    30. multiple_representations?: false (took 1 µs)\n    31. handle_ok: nil (took 3 µs)\n```\n\nLiberator will include the request ID set by the `Plug.RequestId` plug,\nif you have it as part of your pipeline.\n\nSet the `:trace` option to `:headers` so you can get the trace as HTTP headers.\n\n    defmodule MyFirstResource do\n      use Liberator.Resource, trace: :headers\n\n      def handle_ok(_), do: \"Hello world!\"\n    end\n\nThis will add a header called `x-liberator-trace` that will show you the entire set of decisions, in the order they were made.\n\nLastly, Liberator is instrumented with the [Telemetry](https://github.com/beam-telemetry/telemetry) library,\nand emits events upon the completion of every request.\nA Resource will emit the following events:\n\n- `[:liberator, :request, :start]`\n- `[:liberator, :request, :stop]`\n- `[:liberator, :request, :exception]`\n\nAll Telemetry events will contain the request ID and request path as metadata.\nThe measurements for `:start` will contain a key called `system_time` which is derived by calling `:erlang.system_time()`\nThe measurements for `:stop` and `:exception` will both contain a key called `:duration`, which is the duration of the request in native time units.\nThe metadata for the `:stop` event will also contain the execution trace, which is a list of maps with the following keys:\n\n- `:step`: the name of the function that was executed, or the atoms `:start` or `:stop`\n- `:result`: the value the function returned\n- `:timestamp`: the time the function was called, as an Elixir date struct\n- `:duration`: how long the call took, in native time units","ref":"Liberator.Resource.html#module-debugging"},{"type":"behaviour","title":"Internationalization and Localization (i18n and l10n) - Liberator.Resource","doc":"During content negotiation (specifically, in the call to `c:language_available?/1`)\nthe `accept-language` header is inspected, and the `:language` key is added to `conn.assigns`.\nThis callback also sets the `Gettext` locale for the current process to the value it finds, using `Gettext.put_locale/1`.\nSo, in your handler functions, all you need to do is make your usual `gettext` calls.\nEverything else is handled for you.\n\n    defmodule MyInternationalizedResource do\n      use Liberator.Resource\n\n      def available_languages(_): [\"en\", \"es\", \"de\", \"fr\"]\n      def handle_ok(_), do: gettext(\"Hello world!\")\n    end\n\nAll of the default handler messages are internationalized, but we need help with translations!\nIf you can help with this, please submit a pull request on [Liberator's GitHub page](https://github.com/Cantido/liberator)!","ref":"Liberator.Resource.html#module-internationalization-and-localization-i18n-and-l10n"},{"type":"behaviour","title":"Advanced Overrides - Liberator.Resource","doc":"Liberator tries to give you access to as much of the program as possible.\nLots of the guts are open for you to play around in.\nYou probably won't ever need to mess with this stuff,\nbut it's there if you need it.","ref":"Liberator.Resource.html#module-advanced-overrides"},{"type":"behaviour","title":"Overriding Decisions - Liberator.Resource","doc":"These decision points are used internally by Liberator and provide reasonable defaults.\nOverriding is possible, but not useful in general.\n\n| Function                                 | Description                                                                    |\n|---                                       |---                                                                             |\n| `c:accept_charset_exists?/1`             | Checks if header `Accept-Charset` exists.                                      |\n| `c:accept_encoding_exists?/1`            | Checks if header `Accept-Encoding` exists.                                     |\n| `c:accept_exists?/1`                     | Checks if header `Accept` exists.                                              |\n| `c:accept_language_exists?/1`            | Checks if header `Accept-Language` exists.                                     |\n| `c:body_exists?/1`                       | Checks if the request has a body.                                              |\n| `c:if_match_exists?/1`                   | Checks if header `If-Match` exists.                                            |\n| `c:if_match_star?/1`                     | Checks if header `If-Match` is `*`.                                            |\n| `c:if_match_star_exists_for_missing?/1`  | Checks if header `If-Match` exists for a resource that does not exist.         |\n| `c:if_modified_since_exists?/1`          | Checks if header `If-Modified-Since` exists.                                   |\n| `c:if_modified_since_valid_date?/1`      | Checks if header `If-Modified-Since` is a valid HTTP date.                     |\n| `c:if_none_match?/1`                     | Checks if the request method to handle failed `If-None-Match`                  |\n| `c:if_none_match_exists?/1`              | Checks if header `If-None-Match` exists.                                       |\n| `c:if_none_match_star?/1`                | Checks if header `If-None-Match` is `*`.                                       |\n| `c:if_unmodified_since_exists?/1`        | Checks if header `If-Unmodified-Since` exists.                                 |\n| `c:if_unmodified_since_valid_date?/1`    | Checks if header `If-Unmodified-Since` is a valid HTTP date.                   |\n| `c:method_options?/1`                        | Checks if the request method is `OPTIONS`                                      |\n| `c:method_delete?/1`                     | Checks if the request method is `DELETE`                                       |\n| `c:method_put?/1`                        | Checks if the request method is `PUT`                                          |\n| `c:method_patch?/1`                      | Checks if the request method is `PATCH`                                        |\n| `c:post_to_gone?/1`                      | Checks if the request method is `POST` for resources that do not exist anymore.|\n| `c:post_to_existing?/1`                  | Checks if the request method is `POST` for resources that do exist.            |\n| `c:post_to_missing?/1`                   | Checks if the request method is `POST` for resources that do not exist.        |\n| `c:put_to_existing?/1`                   | Checks if the request method is `PUT` for a resource that exists.              |","ref":"Liberator.Resource.html#module-overriding-decisions"},{"type":"behaviour","title":"Adding Decisions - Liberator.Resource","doc":"Since version 1.3, you can even override the decision, handler, and action trees.\nTo override the decision tree, add an option named `:decision_tree_overrides` into your `use` statement.\nThe decision tree is a map of `atom -> {atom, atom}`,\nwhere all three atoms should be function names in the module that called `use`.\nThe first element of the tuple is the next function to call if the key function returns `true`,\nand the second element of the tuple is the function to call if the function returns `false`.\nYour argument to `:decision_tree_overrides` will be merged into the default decision tree.\nFor example, here's me overriding the first chunk of the decision tree so that the decision `c:uri_too_long?/1` is completely skipped.\nThat decisions happens right after `c:known_method?/1`, so just update that key to call the next decision instead, which is `c:method_allowed?/1`\n\n    defmodule LongUrisResource do\n      use Liberator.Resource,\n        decision_tree_overrides:  %{\n          # instead of known_method?: {:uri_too_long?, :handle_unknown_method}\n          known_method?: {:method_allowed?, :handle_unknown_method}\n        }\n    end\n\nEvery function in the decision matrix needs an entry.\nIf you're adding a new decision function of your own,\nthat new decision needs to be in both a result tuple and a key.\nOtherwise, Liberator will throw an exception.\nAlso note that Liberator cannot detect a cycle in your callbacks,\nso be careful!","ref":"Liberator.Resource.html#module-adding-decisions"},{"type":"behaviour","title":"Adding Handlers - Liberator.Resource","doc":"To override the handler status, or add your own,\nadd an option named `:handler_status_overrides` to your `use` statement,\nwith a map of `atom -> integer`.\nThe integers are the status codes that Liberator will set before calling the actual handler function.\n\nIf you are adding a new status code to Liberator,\nyou'll also need to set `:decision_tree_overrides` in order to actually call this new handler,\nas well as a functions of those names defined in the module that called `use`.\nHere's an example of adding a handler for a new status code:\n\n    defmodule ResourceLikesToParty do\n      use Liberator.Resource,\n        decision_tree_overrides:  %{\n          allowed?: {:likes_to_party?, :handle_forbidden}\n          likes_to_party?: {:handle_likes_to_party, :too_many_requests?}\n        },\n        handler_status_overrides: %{\n          handle_likes_to_party: 420\n        }\n\n      def likes_to_party?(_conn), do: Enum.random([true, false])\n      def handle_likes_to_party(_conn), do: \"Hey come party with me sometime.\"\n    end\n\nIn this example, the `likes_to_party?/1` callback is added,\nand if that function returns `false`, it will continue on with the pipeline,\nbut if it returns `true`, then it will call the new `handle_likes_to_party/1` callback,\nand set the status code to 420.","ref":"Liberator.Resource.html#module-adding-handlers"},{"type":"behaviour","title":"Adding Actions - Liberator.Resource","doc":"Finally, you can override actions as well.\nThe option is called `:action_followup_overrides`,\nand is a map of `atom -> atom`,\nwhere both atoms are functions defined in the module that called `use`.\nThe first atom is the name of the handler function, like `c:post!/1` or `c:delete!/1`.\nThe second atom is the function that will be called immediately after the action.\n\nSay you're implementing a [WebDAV](https://en.wikipedia.org/wiki/WebDAV) server using Liberator,\nand you want to add your own `COPY` decision, action, and handler.\nBy overriding some internals, this is how you'd do it,\nand still have the power of the decision tree on your side!\n\n    defmodule WebDavResource do\n      use Liberator.Resource,\n        decision_tree_overrides:  %{\n          method_delete?: {:delete!, :method_copy?}\n          method_copy?: {:lock_token_valid?, :method_patch?}\n          lock_token_valid?: {:copy!, :handle_locked}\n        },\n        handler_status_overrides: %{\n          handle_locked: 423\n        },\n        action_followup_overrides: %{\n          copy!: :respond_with_entity?\n        }\n      }\n\n      @impl true\n      def available_media_types(_), do: [\"application/xml\"]\n\n      @impl true\n      def allowed_methods(_), do: [\"COPY\"]\n\n      @impl true\n      def known_methods(_), do: [\"GET\", \"HEAD\", \"OPTIONS\", \"PUT\", \"POST\", \"DELETE\", \"PATCH\", \"TRACE\", \"COPY\"]\n\n      def lock_token_valid?(conn), do: MyWebDavBackend.lock_token_valid?(conn)\n      def copy!(conn), do: MyWebDavBackend.copy(conn)\n      def handle_locked(_conn), do: \"Resource Locked\"\n    end","ref":"Liberator.Resource.html#module-adding-actions"},{"type":"callback","title":"Liberator.Resource.accept_charset_exists?/1","doc":"Check if the `Accept-Charset` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_charset_exists?/1"},{"type":"callback","title":"Liberator.Resource.accept_encoding_exists?/1","doc":"Check if the `Accept-Encoding` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_encoding_exists?/1"},{"type":"callback","title":"Liberator.Resource.accept_exists?/1","doc":"Check if the `Accept` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_exists?/1"},{"type":"callback","title":"Liberator.Resource.accept_language_exists?/1","doc":"Check if the `Accept-Language` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_language_exists?/1"},{"type":"callback","title":"Liberator.Resource.allowed?/1","doc":"Check the authentication information in the request to see if it has the necessary permissions.\n\nNote the difference between `c:authorized?/1` and `c:allowed?/1`.\nThis function checks if the given request is allowed to perform an action,\nbut isn't responsible for checking the presence of authentication information in the first place.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:allowed?/1"},{"type":"callback","title":"Liberator.Resource.allowed_methods/1","doc":"Returns a list of HTTP methods that this module serves.\n\nThe methods returned by this function should be upper-case strings, like `\"GET\"`, `\"POST\"`, etc.","ref":"Liberator.Resource.html#c:allowed_methods/1"},{"type":"callback","title":"Liberator.Resource.authorized?/1","doc":"Check for presence ofauthentication information in the request.\n\nNote the difference between `c:authorized?/1` and `c:allowed?/1`.\nThis function should just check for the presence of authentication information,\nnot the content of it.\n\nIf you implement this function to return `false`, your response in `handle_unauthorized`\nmust include a `WWW-Authenticate` header field containing a challenge applicable to the requested resource.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:authorized?/1"},{"type":"callback","title":"Liberator.Resource.available_charsets/1","doc":"Returns a list of available content charsets.\n\nBy default, only `UTF-8` is supported.","ref":"Liberator.Resource.html#c:available_charsets/1"},{"type":"callback","title":"Liberator.Resource.available_encodings/1","doc":"Returns a list of available response content encodings (AKA compressions).\n\nBy default, only `identity` (no compression) is supported.","ref":"Liberator.Resource.html#c:available_encodings/1"},{"type":"callback","title":"Liberator.Resource.available_languages/1","doc":"Returns a list of available languages.","ref":"Liberator.Resource.html#c:available_languages/1"},{"type":"callback","title":"Liberator.Resource.available_media_types/1","doc":"Returns a list of content types that this module serves.\n\nThe types returned by this function should be valid MIME types, like `text/plain`, `application/json`, etc.","ref":"Liberator.Resource.html#c:available_media_types/1"},{"type":"callback","title":"Liberator.Resource.body_exists?/1","doc":"Check if the request has a body.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:body_exists?/1"},{"type":"callback","title":"Liberator.Resource.can_post_to_gone?/1","doc":"Decide if we can process a `POST` to a resource that existed before, or return a 410 Gone response.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:can_post_to_gone?/1"},{"type":"callback","title":"Liberator.Resource.can_post_to_missing?/1","doc":"Check if we can process a post to a resource that does not exist,\nor if we should send a 404 Not Found response.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:can_post_to_missing?/1"},{"type":"callback","title":"Liberator.Resource.can_put_to_missing?/1","doc":"Decide if we can `PUT` to a resource that does not exist, or return 501 Not Implemented.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:can_put_to_missing?/1"},{"type":"callback","title":"Liberator.Resource.charset_available?/1","doc":"Check of the requested charset is available.\n\nBy default, uses the values returned by `c:available_charsets/1`,\nand returns a map with the key `:charset` set to the negotiated charset,\nwhich will be merged into `conn.assigns`.","ref":"Liberator.Resource.html#c:charset_available?/1"},{"type":"callback","title":"Liberator.Resource.conflict?/1","doc":"Does the `PUT` or `POST` request result in a conflict?","ref":"Liberator.Resource.html#c:conflict?/1"},{"type":"callback","title":"Liberator.Resource.delete!/1","doc":"Called for `DELETE` requests.","ref":"Liberator.Resource.html#c:delete!/1"},{"type":"callback","title":"Liberator.Resource.delete_enacted?/1","doc":"Check if the `DELETE` request was processed.\nReturn `false` here if the request was put on some processing queue and the\ndelete was not actually enacted yet.\nReturning `false` here would return a 202 Accepted instead of some other response.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:delete_enacted?/1"},{"type":"callback","title":"Liberator.Resource.encoding_available?/1","doc":"Check of the requested encoding is available.\n\nBy default, uses the values returned by `c:available_encodings/1`,\nand returns a map with the key `:encoding` set to the negotiated encoding,\nwhich will be merged into `conn.assigns`.","ref":"Liberator.Resource.html#c:encoding_available?/1"},{"type":"callback","title":"Liberator.Resource.etag/1","doc":"Returns the etag for the current entity.\n\nThis value will be used to respond to caching headers like `If-None-Match`.","ref":"Liberator.Resource.html#c:etag/1"},{"type":"callback","title":"Liberator.Resource.etag_matches_for_if_match?/1","doc":"Check if the etag for the current resource matches the value in the `If-Match` header.\n\nBy default, checks the header against the value returned by `c:etag/1`.","ref":"Liberator.Resource.html#c:etag_matches_for_if_match?/1"},{"type":"callback","title":"Liberator.Resource.etag_matches_for_if_none?/1","doc":"Check if the etag of the current resource matches the `If-Match-None` header.\n\nBy default, checks the header against the value returned by `c:etag/1`.","ref":"Liberator.Resource.html#c:etag_matches_for_if_none?/1"},{"type":"callback","title":"Liberator.Resource.existed?/1","doc":"Check if the resource ever existed.\n\nAnswering `true` here will lead you down the path that leads to\nresponses like \"Moved Permanently\" and \"Gone\", among othes.","ref":"Liberator.Resource.html#c:existed?/1"},{"type":"callback","title":"Liberator.Resource.exists?/1","doc":"Check if the requested entity exists.\n\nThis is a great place to actually fetch the requested resource,\nthen return it as a map so it can be merged into the `:assigns` map of the request.\n\nReturning `false` here will cause the plug to return a 404 Not Found response.","ref":"Liberator.Resource.html#c:exists?/1"},{"type":"callback","title":"Liberator.Resource.handle_accepted/1","doc":"Returns content for a `202 Accepted` response.","ref":"Liberator.Resource.html#c:handle_accepted/1"},{"type":"callback","title":"Liberator.Resource.handle_conflict/1","doc":"Returns content for a `409 Conflict` response.","ref":"Liberator.Resource.html#c:handle_conflict/1"},{"type":"callback","title":"Liberator.Resource.handle_created/1","doc":"Returns content for a `201 Created` response.","ref":"Liberator.Resource.html#c:handle_created/1"},{"type":"callback","title":"Liberator.Resource.handle_error/3","doc":"Returns a conn for a `500 Internal Server Error` response.\n\nThis handler is special among the handlers.\nNot only does it receive the `conn`, it also receives the error that was thrown,\nas well as the atom name of the failed step.","ref":"Liberator.Resource.html#c:handle_error/3"},{"type":"callback","title":"Liberator.Resource.handle_forbidden/1","doc":"Returns content for a `403 Forbidden` response.","ref":"Liberator.Resource.html#c:handle_forbidden/1"},{"type":"callback","title":"Liberator.Resource.handle_gone/1","doc":"Returns content for a `410 Gone` response.","ref":"Liberator.Resource.html#c:handle_gone/1"},{"type":"callback","title":"Liberator.Resource.handle_malformed/1","doc":"Returns content for a `400 Malformed` response.","ref":"Liberator.Resource.html#c:handle_malformed/1"},{"type":"callback","title":"Liberator.Resource.handle_method_not_allowed/1","doc":"Returns content for a `405 Method Not Allowed` response.","ref":"Liberator.Resource.html#c:handle_method_not_allowed/1"},{"type":"callback","title":"Liberator.Resource.handle_moved_permanently/1","doc":"Returns content for a `301 Moved Permanently` response.","ref":"Liberator.Resource.html#c:handle_moved_permanently/1"},{"type":"callback","title":"Liberator.Resource.handle_moved_temporarily/1","doc":"Returns content for a `307 Moved Permanently` response.","ref":"Liberator.Resource.html#c:handle_moved_temporarily/1"},{"type":"callback","title":"Liberator.Resource.handle_multiple_representations/1","doc":"Returns content for a `300 Multiple Representations` response.","ref":"Liberator.Resource.html#c:handle_multiple_representations/1"},{"type":"callback","title":"Liberator.Resource.handle_no_content/1","doc":"Returns content for a `204 No Content` response.","ref":"Liberator.Resource.html#c:handle_no_content/1"},{"type":"callback","title":"Liberator.Resource.handle_not_acceptable/1","doc":"Returns content for a `406 Not Acceptable` response.","ref":"Liberator.Resource.html#c:handle_not_acceptable/1"},{"type":"callback","title":"Liberator.Resource.handle_not_found/1","doc":"Returns content for a `404 Not Found` response.","ref":"Liberator.Resource.html#c:handle_not_found/1"},{"type":"callback","title":"Liberator.Resource.handle_not_implemented/1","doc":"Returns content for a `501 Not Implemented` response.","ref":"Liberator.Resource.html#c:handle_not_implemented/1"},{"type":"callback","title":"Liberator.Resource.handle_not_modified/1","doc":"Returns content for a `304 Not Modified` response.","ref":"Liberator.Resource.html#c:handle_not_modified/1"},{"type":"callback","title":"Liberator.Resource.handle_ok/1","doc":"Returns content for a `200 OK` response.","ref":"Liberator.Resource.html#c:handle_ok/1"},{"type":"callback","title":"Liberator.Resource.handle_options/1","doc":"Returns content for a `200 OK` response to an `OPTIONS` request.","ref":"Liberator.Resource.html#c:handle_options/1"},{"type":"callback","title":"Liberator.Resource.handle_payment_required/1","doc":"Returns content for a `402 Payment Required` response.\n\nPlease note that the 402 status code is experimental, and is \"reserved for future use.\"","ref":"Liberator.Resource.html#c:handle_payment_required/1"},{"type":"callback","title":"Liberator.Resource.handle_precondition_failed/1","doc":"Returns content for a `412 Precondition Failed` response.","ref":"Liberator.Resource.html#c:handle_precondition_failed/1"},{"type":"callback","title":"Liberator.Resource.handle_request_entity_too_large/1","doc":"Returns content for a `413 Entity Too Large` response.","ref":"Liberator.Resource.html#c:handle_request_entity_too_large/1"},{"type":"callback","title":"Liberator.Resource.handle_see_other/1","doc":"Returns content for a `303 See Other` response.","ref":"Liberator.Resource.html#c:handle_see_other/1"},{"type":"callback","title":"Liberator.Resource.handle_service_unavailable/1","doc":"Returns content for a `503 Service Unavailable` response.","ref":"Liberator.Resource.html#c:handle_service_unavailable/1"},{"type":"callback","title":"Liberator.Resource.handle_too_many_requests/1","doc":"Returns content for a `429 Too Many Requests` response.\n\nFor more information on this response type, see [RFC 6585, section 4](https://tools.ietf.org/html/rfc6585#section-4).","ref":"Liberator.Resource.html#c:handle_too_many_requests/1"},{"type":"callback","title":"Liberator.Resource.handle_unauthorized/1","doc":"Returns content for a `401 Unauthorized` response.","ref":"Liberator.Resource.html#c:handle_unauthorized/1"},{"type":"callback","title":"Liberator.Resource.handle_unavailable_for_legal_reasons/1","doc":"Returns content for a `451 Unavailable for Legal Reasons` response.\n\nFor more information on this response type, see [RFC 7725](https://tools.ietf.org/html/rfc7725).","ref":"Liberator.Resource.html#c:handle_unavailable_for_legal_reasons/1"},{"type":"callback","title":"Liberator.Resource.handle_unknown_method/1","doc":"Returns content for a `501 Unknown Method` response.","ref":"Liberator.Resource.html#c:handle_unknown_method/1"},{"type":"callback","title":"Liberator.Resource.handle_unprocessable_entity/1","doc":"Returns content for a `422 Unprocesable Entity` response.","ref":"Liberator.Resource.html#c:handle_unprocessable_entity/1"},{"type":"callback","title":"Liberator.Resource.handle_unsupported_media_type/1","doc":"Returns content for a `415 Unsuppported Media Type` response.","ref":"Liberator.Resource.html#c:handle_unsupported_media_type/1"},{"type":"callback","title":"Liberator.Resource.handle_uri_too_long/1","doc":"Returns content for a `414 URI Too Long` response.","ref":"Liberator.Resource.html#c:handle_uri_too_long/1"},{"type":"callback","title":"Liberator.Resource.if_match_exists?/1","doc":"Check if the `If-Match` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_exists?/1"},{"type":"callback","title":"Liberator.Resource.if_match_star?/1","doc":"Check if the `If-Match` header is  `*`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_star?/1"},{"type":"callback","title":"Liberator.Resource.if_match_star_exists_for_missing?/1","doc":"Check if the `If-Match *` header exists for a resource that does not exist.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_star_exists_for_missing?/1"},{"type":"callback","title":"Liberator.Resource.if_modified_since_exists?/1","doc":"Check if the `If-Modified-Since` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_modified_since_exists?/1"},{"type":"callback","title":"Liberator.Resource.if_modified_since_valid_date?/1","doc":"Check if the `If-Modified-Since` header is a valid HTTP date.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_modified_since_valid_date?/1"},{"type":"callback","title":"Liberator.Resource.if_none_match?/1","doc":"Check if the request method to handle failed if-none-match.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match?/1"},{"type":"callback","title":"Liberator.Resource.if_none_match_exists?/1","doc":"Check if the `If-None-Match` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match_exists?/1"},{"type":"callback","title":"Liberator.Resource.if_none_match_star?/1","doc":"Check if the `If-None-Match` header is `*`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match_star?/1"},{"type":"callback","title":"Liberator.Resource.if_unmodified_since_exists?/1","doc":"Check if the `If-Unmodified-Since` header exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_unmodified_since_exists?/1"},{"type":"callback","title":"Liberator.Resource.if_unmodified_since_valid_date?/1","doc":"Check if the `If-Unmodified-Since` header is a valid HTTP date.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_unmodified_since_valid_date?/1"},{"type":"callback","title":"Liberator.Resource.initialize/1","doc":"A hook invoked at the beginning of the decision tree to set up anything you may need.\n\nYou can return a map here and it will be merged with the given conn's `:assigns` map.","ref":"Liberator.Resource.html#c:initialize/1"},{"type":"callback","title":"Liberator.Resource.known_content_type?/1","doc":"Check if the Content-Type of the body is known.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:known_content_type?/1"},{"type":"callback","title":"Liberator.Resource.known_method?/1","doc":"Check of the HTTP method in the request is one we know about.\n\nThis is different from `c:allowed_methods/1` in that this function\nchecks to see if the given HTTP method is an HTTP method at all.\nYou probably want to override `c:allowed_methods/1` and not this one,\nunless you're extending HTTP with more verbs.\n\nIf this function returns `false`, then the plug will return a 501 Unknown Method response.\n\nBy default, allows the methods returned by `c:known_methods/1`.","ref":"Liberator.Resource.html#c:known_method?/1"},{"type":"callback","title":"Liberator.Resource.known_methods/1","doc":"Returns a list of HTTP methods that exist.\n\nNote that this is to filter bad HTTP requests, not to filter requests that your endpoint does not serve.\nYou probably want to implement `c:allowed_methods/1` instead.\n\nThe methods returned by this function should be upper-case strings, like `\"GET\"`, `\"POST\"`, etc.","ref":"Liberator.Resource.html#c:known_methods/1"},{"type":"callback","title":"Liberator.Resource.language_available?/1","doc":"Check if the requested language is available.\n\nBy default, uses the values returned by `c:available_languages/1`,\nand returns a map with the key `:language` set to the negotiated language,\nwhich will be merged into `conn.assigns`.","ref":"Liberator.Resource.html#c:language_available?/1"},{"type":"callback","title":"Liberator.Resource.last_modified/1","doc":"Returns the last modified date of your resource.\n\nThis value will be used to respond to caching headers like `If-Modified-Since`.","ref":"Liberator.Resource.html#c:last_modified/1"},{"type":"callback","title":"Liberator.Resource.malformed?/1","doc":"Check the request for general adherence to some form.\n\nIf this function returns true, then the plug will return a 400 Malformed response.\n\nIf you're checking the body of a request against some schema,\nyou should override `c:processable?/1` instead.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:malformed?/1"},{"type":"callback","title":"Liberator.Resource.maximum_entity_length/1","doc":"Configures the maximum length that a request body can be.","ref":"Liberator.Resource.html#c:maximum_entity_length/1"},{"type":"callback","title":"Liberator.Resource.media_type_available?/1","doc":"Check if the request media type is available.\n\nBy default, uses the values returned by `c:available_media_types/1`.","ref":"Liberator.Resource.html#c:media_type_available?/1"},{"type":"callback","title":"Liberator.Resource.method_allowed?/1","doc":"Check if the server supports the request's HTTP method.\n\nOverride `c:allowed_methods/1` instead of this function to let this plug perform the check for you.\n\nBy default, allows the methods returned by `c:allowed_methods/1`.","ref":"Liberator.Resource.html#c:method_allowed?/1"},{"type":"callback","title":"Liberator.Resource.method_delete?/1","doc":"Check if the request method is `DELETE`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_delete?/1"},{"type":"callback","title":"Liberator.Resource.method_options?/1","doc":"Check if the request method is Options.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_options?/1"},{"type":"callback","title":"Liberator.Resource.method_patch?/1","doc":"Check if the request method is `PATCH`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_patch?/1"},{"type":"callback","title":"Liberator.Resource.method_post?/1","doc":"Check if the request method is `POST`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_post?/1"},{"type":"callback","title":"Liberator.Resource.method_put?/1","doc":"Check if the request method is `PUT`.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_put?/1"},{"type":"callback","title":"Liberator.Resource.modified_since?/1","doc":"Checks if the resource was modified since the date given in the `If-Modified-Since` header.\n\nBy default, checks the header against the value returned by `c:last_modified/1`.","ref":"Liberator.Resource.html#c:modified_since?/1"},{"type":"callback","title":"Liberator.Resource.moved_permanently?/1","doc":"Check if the resource was moved permanently.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:moved_permanently?/1"},{"type":"callback","title":"Liberator.Resource.moved_temporarily?/1","doc":"Check if the resource was moved temporarily.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:moved_temporarily?/1"},{"type":"callback","title":"Liberator.Resource.multiple_representations?/1","doc":"Check if there are multiple representations of the resource.","ref":"Liberator.Resource.html#c:multiple_representations?/1"},{"type":"callback","title":"Liberator.Resource.new?/1","doc":"Was the resource created by this request?","ref":"Liberator.Resource.html#c:new?/1"},{"type":"callback","title":"Liberator.Resource.patch!/1","doc":"Called for `PATCH` requests.","ref":"Liberator.Resource.html#c:patch!/1"},{"type":"callback","title":"Liberator.Resource.patch_enacted?/1","doc":"Check if the `PATCH` request was processed.\nReturn `false` here if the request was put on some processing queue and the\npatch was not actually enacted yet.\nReturning `false` here would return a 202 Accepted instead of some other response.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:patch_enacted?/1"},{"type":"callback","title":"Liberator.Resource.payment_required?/1","doc":"Check to see if payment is required for this resource.\n\nIf this function returns true, then the plug will return a 402 Payment Required response.\nPlease note that the 402 status code is experimental, and is \"reserved for future use.\"\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:payment_required?/1"},{"type":"callback","title":"Liberator.Resource.post!/1","doc":"Called for `POST` requests.","ref":"Liberator.Resource.html#c:post!/1"},{"type":"callback","title":"Liberator.Resource.post_enacted?/1","doc":"Check if the `POST` request was processed.\nReturn `false` here if the request was put on some processing queue and the\npost was not actually enacted yet.\nReturning `false` here would return a 202 Accepted instead of some other response.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:post_enacted?/1"},{"type":"callback","title":"Liberator.Resource.post_redirect?/1","doc":"Decide if the response should redirect after a `POST`.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:post_redirect?/1"},{"type":"callback","title":"Liberator.Resource.post_to_existing?/1","doc":"Check if the request method is `POST` for a resource that already exists.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_existing?/1"},{"type":"callback","title":"Liberator.Resource.post_to_gone?/1","doc":"Check if the request method is `POST` for resources that do not exist anymore.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_gone?/1"},{"type":"callback","title":"Liberator.Resource.post_to_missing?/1","doc":"Check if the request method is `POST` to a resource that doesn't exist.\n\nUsed internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_missing?/1"},{"type":"callback","title":"Liberator.Resource.processable?/1","doc":"Check if the body of the request can be processed.\n\nThis is a good place to parse a JSON body if that's what you're doing.\nReturning `false` here would cause the plug to return a 422 Unprocessable response.","ref":"Liberator.Resource.html#c:processable?/1"},{"type":"callback","title":"Liberator.Resource.put!/1","doc":"Called for `PUT` requests.","ref":"Liberator.Resource.html#c:put!/1"},{"type":"function","title":"Liberator.Resource.put_action_followup_overrides/2","doc":"","ref":"Liberator.Resource.html#put_action_followup_overrides/2"},{"type":"function","title":"Liberator.Resource.put_decision_tree_overrides/2","doc":"","ref":"Liberator.Resource.html#put_decision_tree_overrides/2"},{"type":"callback","title":"Liberator.Resource.put_enacted?/1","doc":"Check if the `PUT` request was processed.\nReturn `false` here if the request was put on some processing queue and the\nput was not actually enacted yet.\nReturning `false` here would return a 202 Accepted instead of some other response.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:put_enacted?/1"},{"type":"function","title":"Liberator.Resource.put_handler_status_overrides/2","doc":"","ref":"Liberator.Resource.html#put_handler_status_overrides/2"},{"type":"function","title":"Liberator.Resource.put_liberator_module/2","doc":"","ref":"Liberator.Resource.html#put_liberator_module/2"},{"type":"callback","title":"Liberator.Resource.put_to_different_url?/1","doc":"Decide if a `PUT` request should be made to a different URL.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:put_to_different_url?/1"},{"type":"callback","title":"Liberator.Resource.put_to_existing?/1","doc":"Check if the request method is a `PUT` for a resource that already exists.","ref":"Liberator.Resource.html#c:put_to_existing?/1"},{"type":"callback","title":"Liberator.Resource.respond_with_entity?/1","doc":"Should the response contain a representation of the resource?","ref":"Liberator.Resource.html#c:respond_with_entity?/1"},{"type":"callback","title":"Liberator.Resource.service_available?/1","doc":"Check if your service is available.\n\nThis is the first function called in the entire pipeline,\nand lets you check to make sure everything works before going deeper.\nIf this function returns `false`, then the plug will return a 503 Service Not Available response.\n\nIf this function returns a map containing a value called `:retry_after`,\nLiberator will put this value into a `retry-after` header,\nSome crawlers and spiders honor this value,\nso they will not bother you while you're down,\nand will continue to index your site afterward.\nSee [MDN's docs on the `retry-after` header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Retry-After)\nfor more information.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:service_available?/1"},{"type":"callback","title":"Liberator.Resource.too_many_requests?/1","doc":"Check to see if the client has performed too many requests.\nUsed as part of a rate limiting scheme.\n\nIf you return a map containing a `:retry_after` key,\nthen the response's `retry-after` header will be automatically set.\nThe value of this key can be either an Elixir `DateTime` object,\na `String` HTTP date, or an integer of seconds.\nAll of these values tell the client when they can attempt their request again.\nNote that if you provide a `String` for this value,\nit should be formatted as an HTTP date.\n\nIf you do return map with the key `:retry_after` set,\nand its value is not a `DateTime`, integer, or valid `String`,\nthen Liberator will raise an exception.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:too_many_requests?/1"},{"type":"callback","title":"Liberator.Resource.unavailable_for_legal_reasons?/1","doc":"Check if the resource is no longer available, for legal reasons.\n\nIf this function returns `true`,\nthen the plug will return a 451 Unavailable for Legal Reasons response.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:unavailable_for_legal_reasons?/1"},{"type":"callback","title":"Liberator.Resource.unmodified_since?/1","doc":"Checks if the resource was not modified since the date given in the `If-Unmodified-Since` header.\n\nBy default, checks the header against the value returned by `c:last_modified/1`.","ref":"Liberator.Resource.html#c:unmodified_since?/1"},{"type":"callback","title":"Liberator.Resource.uri_too_long?/1","doc":"Checks the length of the URI.\n\nIf this function returns true, then the plug will return a 414 URI Too Long response.\n\nBy default, always returns `false`.","ref":"Liberator.Resource.html#c:uri_too_long?/1"},{"type":"callback","title":"Liberator.Resource.valid_content_header?/1","doc":"Check if the Content-Type of the body is valid.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:valid_content_header?/1"},{"type":"callback","title":"Liberator.Resource.valid_entity_length?/1","doc":"Check if the length of the body is valid.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:valid_entity_length?/1"},{"type":"callback","title":"Liberator.Resource.well_formed?/1","doc":"Check the request for general adherence to some form.\n\nIf this function returns false, then the plug will return a 400 Malformed response.\n\nThis is a good place to parse the request body.\nIf, for example, the body is not valid JSON, this is the function that should return false.\nIf you're checking the body of a request against some schema,\nlike checking if your changeset is valid,\nyou should override `c:processable?/1` instead.\n\nBy default, always returns `true`.","ref":"Liberator.Resource.html#c:well_formed?/1"},{"type":"task","title":"mix liberator.chart","doc":"Generates source text for a decision tree chart for a Liberator resource.\nThe chart is compatible with the [Graphviz](https://graphviz.org/) graph visualization software.\n\n```sh\nmix liberator.chart\n```\n\nBy default, this function will print the default decision tree to standard output.\nThis task can also take a module argument for any module that `use`s `Liberator.Resource`,\nin which case the decision tree for the given module will be printed.\n\n```sh\nmix liberator.chart MyApp.MyResource\n```\n\nYou can also provide the `--output` or `-o` option to print the chart source to a file.\n\n```sh\nmix liberator.chart -o myresource.dot MyApp.MyResource\n```","ref":"Mix.Tasks.Liberator.Chart.html"},{"type":"task","title":"Generating a chart with the returned source code - mix liberator.chart","doc":"Unfortunately, there's not a Graphviz binding for Elixir.\nIf you want to create an actual image of your chart,\nyou will have to install [Graphviz](https://graphviz.org/),\nor use one of its language bindings for another language.\n\nOnce you have installed Graphviz, you can run a command like the following to generate an image\n\n```sh\ndot myresource.dot -Tsvg -o myresource.svg\n```","ref":"Mix.Tasks.Liberator.Chart.html#module-generating-a-chart-with-the-returned-source-code"},{"type":"function","title":"Mix.Tasks.Liberator.Chart.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Liberator.Chart.html#run/1"},{"type":"module","title":"Liberator.Encoding.Deflate","doc":"An implementation of the `deflate` compression used in HTTP.","ref":"Liberator.Encoding.Deflate.html"},{"type":"module","title":"Liberator.Encoding.Gzip","doc":"An implementation of the `gzip` compression used in HTTP.","ref":"Liberator.Encoding.Gzip.html"},{"type":"module","title":"Liberator.Encoding.Identity","doc":"An implementation of the `identity` compression used in HTTP.\nThat is to say, a no-op function that returns its argument.","ref":"Liberator.Encoding.Identity.html"},{"type":"module","title":"Liberator.MediaType.TextPlain","doc":"An implementation of a `text/plain` encoder for HTTP.","ref":"Liberator.MediaType.TextPlain.html"},{"type":"module","title":"Liberator.Trace","doc":"Decision tracing functions.","ref":"Liberator.Trace.html"},{"type":"function","title":"Liberator.Trace.get_trace/1","doc":"Get the log of all decisions made on the given conn.\n\nThe trace is a list of maps, each map corresponding to one call to a decision function.\nEach map has the following keys:\n\n- `:step`: the name of the function that was executed, or the atoms `:start` or `:stop`\n- `:result`: the value the function returned\n- `:timestamp`: the time the function was called\n- `:duration`: how long the call took, in native time units","ref":"Liberator.Trace.html#get_trace/1"},{"type":"function","title":"Liberator.Trace.headers/1","doc":"Get a list of tuples for the `x-liberator-trace` header,\nbased on the given trace.","ref":"Liberator.Trace.html#headers/1"},{"type":"function","title":"Liberator.Trace.log/3","doc":"Log a message containing the given trace,\nalong with its request path and optional request ID.","ref":"Liberator.Trace.html#log/3"},{"type":"exception","title":"Liberator.CompressionCodecInvalidResultException","doc":"A provided compression codec returned an invalid value.\n\nThe `c:Liberator.Encoding.encode!/1` function must return a binary.","ref":"Liberator.CompressionCodecInvalidResultException.html"},{"type":"exception","title":"Liberator.CompressionCodecNotFoundException","doc":"Liberator was unable to find a compression codec for the given encoding.","ref":"Liberator.CompressionCodecNotFoundException.html"},{"type":"exception","title":"Liberator.InvalidLastModifiedValueException","doc":"Liberator was unable to process the given `:last_modified` value into an HTTP date.","ref":"Liberator.InvalidLastModifiedValueException.html"},{"type":"exception","title":"Liberator.InvalidRetryAfterValueException","doc":"Liberator was unable to process the given `:retry_after` value into an HTTP date.","ref":"Liberator.InvalidRetryAfterValueException.html"},{"type":"exception","title":"Liberator.MediaTypeCodecInvalidResultException","doc":"A provided media type codec returned an invalid value.\n\nThe `c:Liberator.MediaType.encode!/1` function must return a binary.","ref":"Liberator.MediaTypeCodecInvalidResultException.html"},{"type":"exception","title":"Liberator.MediaTypeCodecNotFoundException","doc":"Liberator was unable to find a media type codec for the given encoding.","ref":"Liberator.MediaTypeCodecNotFoundException.html"},{"type":"exception","title":"Liberator.UnknownStepException","doc":"Liberator's decision tree evaluator encountered a step that it doesn't know what to do with.","ref":"Liberator.UnknownStepException.html"},{"type":"extras","title":"Liberator","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Liberator\n\n[![Hex.pm](https://img.shields.io/hexpm/v/liberator)](https://hex.pm/packages/liberator/)\n![Hex.pm License](https://img.shields.io/hexpm/l/liberator)\n\n\nAn Elixir library for building applications with HTTP.\n\nLiberator is a port of the [Liberator](https://clojure-liberator.github.io/liberator/) Clojure library\nthat allows you to define a controller that adheres to the HTTP spec by providing just a few pieces of information.\nIt implements a [decision graph] of simple boolean questions that lead your app to the correct HTTP status codes.\n\nWhile Phoenix and Plug make routing easy, they don't do anything with content negotiation,\nor cache management, or existence checks, or anything like that,\nbeyond calling the right controller function based on the HTTP method.\nThere are a lot of decisions to make before returning the right HTTP status code,\nbut Phoenix doesn't give you any additional power to do so.\nLiberator does.","ref":"readme.html"},{"type":"extras","title":"Install - Liberator","doc":"This package is [available in Hex](https://hex.pm/packages/liberator).\nInstall it by adding `liberator` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:liberator, \"~> 1.4.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand can be found online at [https://hexdocs.pm/liberator](https://hexdocs.pm/liberator).","ref":"readme.html#install"},{"type":"extras","title":"Usage - Liberator","doc":"For a basic `GET` endpoint, you can define an entire module in five lines of code.\nTechnically you don't even need to implement these two,\nsince sensible defaults are provided.\n\n```elixir\ndefmodule MyFirstResource do\n  use Liberator.Resource\n\n  def available_media_types(_), do: [\"text/plain\"]\n  def handle_ok(_), do: \"Hello world!\"\nend\n```\n\nIt doesn't look like much, but behind the scenes,\nLiberator navigated a [decision graph] of content negotation, cache management,\nand existence checks before returning 200 OK.\nLiberator finds the best media type you support,\nand automatically encodes your return value.\nJSON is supported out of the box, and any additional types\ncan be provided in a line of the config.\n\n```elixir\n# in config.exs\nconfig :liberator, media_types: %{\n  \"application/json\" => Jason,\n  \"application/xml\" => MyXmlCodec\n}\n\n# in your main body of code\ndefmodule MyJsonOrXmlResource do\n  use Liberator.Resource\n\n  def available_media_types(_), do: [\"application/json\", \"application/xml\"]\n  def handle_ok(_), do: %{message: \"hi!\"}\nend\n```\n\nA Liberator Resource implements the [Plug](https://github.com/elixir-plug/plug) spec,\nso you can forward requests to it in frameworks like Phoenix:\n\n```elixir\nscope \"/api\", MyApp do\n  pipe_through [:api]\n\n  forward \"/resources\", MyFirstResource\nend\n```\n\nYour results from questions are aggregated into the `:assigns` map on the conn,\nso you don't have to access data more than once.\n\n```elixir\ndefmodule MaybeExistingResource do\n  use Liberator.Resource\n\n  def exists?(conn) do\n    case MyApp.Repo.get(MyApp.Post, conn.params[\"id\"]) do\n      nil -> false\n      post -> %{post: post}\n    end\n  end\n  def handle_ok(conn), do: conn.assigns[:post]\nend\n```\n\nSee more in the [Getting Started guide](guides/getting_started.md),\nand in the [documentation for `Liberator.Resource`](https://hexdocs.pm/liberator/Liberator.Resource.html).\nYou can also see an example controller built with Liberator at [the `liberator_example` project](https://codeberg.org/cosmicrose/liberator_example/).","ref":"readme.html#usage"},{"type":"extras","title":"Maintainer - Liberator","doc":"This project was developed by [Rosa Richter](https://about.me/rosa.richter).\nYou can get in touch with her on [Keybase.io](https://keybase.io/cantido).","ref":"readme.html#maintainer"},{"type":"extras","title":"Thanks - Liberator","doc":"Thanks to the maintainers of the original Clojure [liberator] project,\n[Philipp Meier] and [Malcolm Sparks], for creating such a handy tool.\nTheir great documentation was an immense help in porting it to Elixir.\nAnd thanks to the maintainers of Erlang's [webmachine](https://github.com/basho/webmachine) for inspiring them!\n\n[Philipp Meier]: https://github.com/ordnungswidrig\n[Malcolm Sparks]: https://github.com/malcolmsparks\n[liberator]: https://github.com/clojure-liberator/liberator\n[webmachine]: https://github.com/basho/webmachine","ref":"readme.html#thanks"},{"type":"extras","title":"Contributing - Liberator","doc":"Questions and pull requests are more than welcome.\nI follow Elixir's tenet of bad documentation being a bug,\nso if anything is unclear, please [file an issue](https://todo.sr.ht/~cosmicrose/liberator) or ask on the [mailing list]!\nIdeally, my answer to your question will be in an update to the docs.\n\nPlease see [CONTRIBUTING.md](CONTRIBUTING.md) for all the details you could ever want about helping me with this project.\n\nNote that this project is released with a Contributor [Code of Conduct].\nBy participating in this project you agree to abide by its terms.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Liberator","doc":"MIT License\n\nCopyright 2024 Rosa Richter\n\nThis program is free software: you can redistribute it and/or modify\nit under the terms of the GNU Affero General Public License as\npublished by the Free Software Foundation, either version 3 of the\nLicense, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU Affero General Public License for more details.\n\nYou should have received a copy of the GNU Affero General Public License\nalong with this program.  If not, see  .\n\n[Code of Conduct]: code_of_conduct.md\n[decision graph]: guides/default_decision_tree.svg\n[mailing list]: https://lists.sr.ht/~cosmicrose/liberator","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [3.0.0] - 2025-05-25","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** The `Liberator.Resource.is_options?/1` has been renamed to `Liberator.Resource.method_options?/1`.\n  This is to adhere to a consistent naming convention that Credo recommends.\n- **BREAKING:** The project is now licensed under the [AGPL 3.0](https://www.gnu.org/licenses/agpl-3.0.en.html) (or later) license.\n  If you wish to use this library, your project must be licensed with a compatible license.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- The app now registers `telemetry` events with `telemetry_registry`\n\n## [2.0.0] - 2021-09-06","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- **BREAKING:** All exceptions have been renamed to add `Exception` to their name.\n  This is to adhere to a consistent naming convention that Credo recommends.\n\n## [1.4.0] - 2021-03-23","ref":"changelog.html#changed-1"},{"type":"extras","title":"Added - Changelog","doc":"#### New Callbacks\n\n- The `handle_error/3` handler has been added.\n  This is a special handler that lets you manipulate the `conn` after an error is raised,\n  so you can choose your status code and body yourself.\n  ([docs](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#module-handling-errors))\n  ([#33](https://github.com/Cantido/liberator/issues/33))\n- The `maximum_entity_length/1` function.\n  This is used by the `valid_entity_length?/1` callback.\n  If the size of the request body is above this size,\n  Liberator will return a status of `413 Request Entity Too Large`.\n  ([docs](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#c:maximum_entity_length/1))\n- Added the `well_formed?/1` function.\n  This function checks the request for general adherence to some form, like if it is valid JSON.\n  It replaces `malformed?/1` so that you can return some data after you check for form.\n  ([docs](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#c:well_formed?/1))\n- Added the `body_exists?/1` function.\n  This is an internal function that, uh, checks to see if the body exists.\n  If it does, it'll grab it, and call `valid_entity_length?/1` and\n  the new `well_formed?/1` function.\n  So now you can parse the body if it's there,\n  without worrying about conditional logic if it's not there.\n  ([docs](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#c:body_exists?/1)\n\n#### Additions to callbacks\n\n- You can now return the given conn in decision functions and actions to serve as an affirmative response,\n  as an alternative to returning a plain map, or `true`.\n  Now you can modify the conn as you see fit.\n- You can now return `:ok` or `:error` tuples from decisions, actions, and handlers.\n  Returning `{:error, term}` will invoke the new `handle_error/3` function.\n\n[Docs for callbacks](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#module-decisions)\n\n#### Added internationalization\n\n- Internationalization is now supported via `Gettext`!\n  Liberator already finds the best language for each request,\n  now it also sets the locale using `Gettext.put_locale/1`.\n  Just call `gettext` directly in your Resources.\n  ([docs](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#module-internationalization-and-localization-i18n-and-l10n))\n  ([#8](https://github.com/Cantido/liberator/issues/8),\n  [#10](https://github.com/Cantido/liberator/pull/10))\n\n#### Debugging and tracing upgrades\n\n- Tracing individual step duration is now available.\n  Access the `:duration` key for each decision in the tracing list to get that decision's duration, in native time units.\n- You can now generate a Graphviz file for either the default decision tree or your own resource's decision tree using the `mix liberator.chart` mix task.\n- Telemetry is now sent upon each request.\n  Three events `[:liberator, :request, :start]`, `[:liberator, :request, :stop]`,\n  and `[:liberator, :request, :exception]` are sent.\n  See the docs for `Liberator.Resource` for more information.\n\n[Docs for debugging and tracing](https://hexdocs.pm/liberator/1.4.0/Liberator.Resource.html#module-debugging)","ref":"changelog.html#added-1"},{"type":"extras","title":"Changed - Changelog","doc":"- The `Vary` header is now served by default,\n  with a value of `Accept-Encoding` and `Accept-Language`.\n- Now serves the `location` header if you've returned a map with the `:location` key, or assigned it on the conn.\n- Some decision functions were rearranged.\n- Raised errors are now wrapped in custom exceptions.\nThey're the same errors with the same messages, just with different wrapper types.\n- Entries in the trace list have changed from tuples to maps in order to support more tracing information.\nThe step is now a map member named `:step`, and the value is `:value`","ref":"changelog.html#changed-2"},{"type":"extras","title":"Deprecated - Changelog","doc":"- The `malformed?/1` is now deprecated, use `well_formed?/1` instead.\n  This lets that decision function return data,\n  and it's the ideal place for parsing the body. ([#15](https://github.com/Cantido/liberator/issues/15))","ref":"changelog.html#deprecated"},{"type":"extras","title":"Fixed - Changelog","doc":"- The `etag` header is now included in the response, if you have overridden it.\n  ([#17](https://github.com/Cantido/liberator/issues/17))\n- Now serves the `last-modified` header based on the return value from the `last_modified/1` callback.\n  ([#18](https://github.com/Cantido/liberator/issues/18))\n- Non-printable-`String` return values from handlers will now be passed through `inspect/1` when the content type is `text/plain`.\n  Printable strings will be passed through without hassle. ([#7](https://github.com/Cantido/liberator/issues/7))\n- All responses now include an `allow` header, fixing the cases where one was required but not provided,\n  like in an `OPTIONS` request, and when returning a 405 Method Not Allowed.\n  ([#9](https://github.com/Cantido/liberator/issues/9), [#12](https://github.com/Cantido/liberator/issues/12))\n\n## [1.3.0] - 2020-10-13","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- The `:log` argument to the `:trace` option.\n  Add a `trace: :log` option to your module's `use` statement,\n  and every request will be logged with a trace.\n  The log message even includes a request ID from `Plug.RequestId`, if you have one. ([#4](https://github.com/Cantido/liberator/issues/4))\n- The `:decision_tree_overrides` argument to `Liberator.Resource`.\n  This is an advanced feature that lets users override parts of the decision tree that Liberator evaluates.\n  Want to change the ordering of some decisions, or add new ones?\n  This is the way! ([#5](https://github.com/Cantido/liberator/issues/5))\n- The `:handler_status_overrides` argument to `Liberator.Resource`.\n  This is an advanced feature that lets users override status codes associated with handlers.\n  It's mostly useful for those wanting to implement their own new statuses that Liberator doesn't support.\n- The `:action_followup_overrides` argument to `Liberator.Resource`.\n  This is an advanced feature that lets users override the functions called immediately after actions.\n  It's mostly useful for those wanting to implement their own new actions that Liberator doesn't support.\n\n## [1.2.0] - 2020-10-12","ref":"changelog.html#added-2"},{"type":"extras","title":"Added - Changelog","doc":"- The `402 Payment Required` status,\n  along with related callbacks `payment_required?` and\n  `handle_payment_required/1`.\n- The `451 Unavailable for Legal Reasons` status,\n  along with related callbacks `unavailable_for_legal_reasons?/1` and\n  `handle_unavailable_for_legal_reasons/1`.\n- The `429 Too Many Requests` status,\n  along with related callbacks `too_many_requests?/1` and\n  `handle_too_many_requests/1`.\n  If you return a map containing a `:retry_after` value,\n  Liberator will use that to set a `retry-after` header.\n- You can also return a `:retry_after` value from any other decision function,\n  like `service_available?/1`, or `moved_permanently?/1`, for the same effect.\n  See [MDN's docs on the `retry-after` header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Retry-After)\n  for more information on why you'd want to do this.","ref":"changelog.html#added-3"},{"type":"extras","title":"Fixed - Changelog","doc":"- Dates in headers are now parsed properly. ([#1](https://github.com/Cantido/liberator/issues/1))\n\n## [1.1.0] - 2020-10-04","ref":"changelog.html#fixed-1"},{"type":"extras","title":"Added - Changelog","doc":"- This changelog!\n- The `:trace` option:\n  Add `trace: :headers` to your `use Liberator.Resource` statement to\n  get an `x-liberator-trace` header added to responses,\n  and see the result of all decisions.\n- Compression options: `deflate`, `gzip`, and `identity`.","ref":"changelog.html#added-4"},{"type":"extras","title":"Changed - Changelog","doc":"- Codecs are now configurable.\n  Set the `:media_types` and `:encodings` map in Liberator's config to add your own codecs.","ref":"changelog.html#changed-3"},{"type":"extras","title":"Removed - Changelog","doc":"- `Liberator.Resource` no longer calls `use Timex`, so your context is less polluted.","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Better wildcard handling during content negotiation\n- Content negotiation actually obeys q-values for priority\n\n## [1.0.0] - 2020-10-02","ref":"changelog.html#fixed-2"},{"type":"extras","title":"Added - Changelog","doc":"- Basic decision tree navigation\n\n\n[Unreleased]: https://git.sr.ht/~cosmicrose/liberator/log\n[2.0.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v2.0.0\n[1.4.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v1.4.0\n[1.3.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v1.3.0\n[1.2.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v1.2.0\n[1.1.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v1.1.0\n[1.0.0]: https://git.sr.ht/~cosmicrose/liberator/refs/v1.0.0","ref":"changelog.html#added-5"},{"type":"extras","title":"Contributing","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Contributing\n\nThank you for considering contributing!\nI write libraries in my free time and contributions from others help me make great tools.\n\nFollowing these guidelines helps to communicate that you respect my time,\nas the developer managing and developing this open source project.\nIn return, I should reciprocate that respect in addressing your issue,\nassessing changes, and helping you finalize your pull requests.\n\nQuestions and pull requests are more than welcome.\nI follow Elixir's tenet of bad documentation being a bug,\nso if anything is unclear, please [file an issue](https://github.com/Cantido/liberator/issues/new)!\nIdeally, my answer to your question will be in an update to the docs.\n\nPlease note that this project is released with a Contributor [Code of Conduct]. By participating in this project you agree to abide by its terms.","ref":"contributing.html"},{"type":"extras","title":"Ground Rules - Contributing","doc":"- Follow the [Code of Conduct].\n- Keep your commits clear and your pull requests small.\n  This isn't a big library.\n- Run `mix format` on your files before committing them.\n  I like clean diffs.\n- Run `mix credo` and resolve anything of yours that comes up.\n- Delegate as much as possible to the user, or let them override everything.\n- Provide sensible defaults.\n  If you can `use Liberator.Resource` in an empty module in an empty Phoenix project,\n  and make a request that returns `200 OK`,\n  you're doing great.\n- Don't pollute the user's context.","ref":"contributing.html#ground-rules"},{"type":"extras","title":"Your First Contribution - Contributing","doc":"- Ask questions!\nI like writing good documentation, and questions make that work more meaningful.\nUse the [mailing list] for questions or comments.\nI promise I'll respond promptly.\n- Tests are always welcome!\nLiberator is a project with a lot of conditional logic,\nwith a lot of resulting complexity.\n- Searching issues or pull requests tagged \"[help wanted]\" or \"[good first issue]\" are great places to get started.","ref":"contributing.html#your-first-contribution"},{"type":"extras","title":"Getting Started - Contributing","doc":"If you're on GitHub, it's really easy to submit a pull request. Just:\n\n1. Create your own fork of the code\n2. Do the changes in your fork\n3. [Submit a pull request](https://github.com/Cantido/liberator/compare)\n\nI don't require a CLA or anything like that.\n\nYou can also email a patch to the [mailing list] if you like things old-school.","ref":"contributing.html#getting-started"},{"type":"extras","title":"How report a bug - Contributing","doc":"If you find a security vulnerability, do NOT open an issue.\nContact [Rosa on keybase.io](https://keybase.io/cantido) instead!\n\nIn order to determine whether you are dealing with a security issue, ask yourself these two questions:\n\n- Can I access something that's not mine, or something I shouldn't have access to?\n- Can I disable something for other people?\n\nIf the answer to either of those two questions are \"yes\",\nthen you're probably dealing with a security issue.\nNote that even if you answer \"no\" to both questions,\nyou may still be dealing with a security issue, so if you're unsure,\n[message me directly](https://keybase.io/cantido).\nFor more information on security, see [SECURITY.md](SECURITY.md).\n\nOtherwise, please [open an issue](https://todo.sr.ht/~cosmicrose/liberator).","ref":"contributing.html#how-report-a-bug"},{"type":"extras","title":"How to suggest a feature or enhancement - Contributing","doc":"The philosopy of this project is to create a small, sharp tool that takes the burden\nof the HTTP spec off of the shoulders of developers.\nFeature requests and enhancements should stick to this philosophy.\nHelp me implement the HTTP spec more closely,\nor help me do anything that lets developers do more with their valuable time.\n\nFeature requests give meaning to my work.\n[Open an issue](https://todo.sr.ht/~cosmicrose/liberator) that decribes the feature you'd like to see,\nwhy you need it, and how you'd like it to work.","ref":"contributing.html#how-to-suggest-a-feature-or-enhancement"},{"type":"extras","title":"Code review process - Contributing","doc":"As the primary developer, I will be the one reviewing all pull requests.\nI'm online far too much, so you should be able to hear back from me quickly.\nHowever, I only do this in my free time, so please allow me flexibility.","ref":"contributing.html#code-review-process"},{"type":"extras","title":"Community - Contributing","doc":"Participate in our [mailing list](https://lists.sr.ht/~cosmicrose/liberator)!\nThis is a small project, but I care about the people that care about my work.\nI'm also on [keybase](https://keybase.io/cantido) if you need to message me directly.\n\n---\nIf you like this contribution guide, please give a star to [Nadia Eghbal]'s [`contributing-template`] project on GitHub.\nIt was a great help.\n\n[Nadia Eghbal]: https://github.com/nayafia\n[`contributing-template`]: https://github.com/nayafia/contributing-template\n[mailing list]: https://lists.sr.ht/~cosmicrose/liberator\n[help wanted]: https://todo.sr.ht/~cosmicrose/liberator?search=label%3A\"help wanted\"\n[good first issue]: https://todo.sr.ht/~cosmicrose/liberator?search=label%3A\"help wanted\"\n[Code of Conduct]: code_of_conduct.md","ref":"contributing.html#community"},{"type":"extras","title":"Security Policy","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Security Policy","ref":"security.html"},{"type":"extras","title":"Supported Versions - Security Policy","doc":"All versions of Liberator are currently supported with security updates.\n\n| Version | Supported          |\n| ------- | ------------------ |\n| 1.3.x   | :white_check_mark: |\n| 1.2.x   | :white_check_mark: |\n| 1.1.x   | :white_check_mark: |\n| 1.0.x   | :white_check_mark: |","ref":"security.html#supported-versions"},{"type":"extras","title":"Reporting a Vulnerability - Security Policy","doc":"If you find a security vulnerability, do _not_ open an issue.\nDirectly contact [Rosa on keybase.io](https://keybase.io/cantido) instead.\nHere is the [signed proof of ownership](https://gist.github.com/Cantido/a76b9900c3b579020d3c)\nthat proves both the GitHub and Keybase accounts are owned by the same person.\n\nYou can also send an encrypted email to the primary UID of the PGP key [`60BA2A8BFD583641`].\nThe signatures on Keybase were made with that key.\n\nSecurity fixes are my top priority, and I'll do my best to get it fixed within the day.\nIf not, expect daily personal updates until it's fixed\n\n[`60BA2A8BFD583641`]: https://keybase.io/cantido/pgp_keys.asc?fingerprint=dd2c4f195e70a0d92f862d6f60ba2a8bfd583641","ref":"security.html#reporting-a-vulnerability"},{"type":"extras","title":"Contributor Covenant Code of Conduct","doc":"<!--\nSPDX-FileCopyrightText: 2021 Coraline Ada Ehmke\n\nSPDX-License-Identifier: CC-BY-4.0\n-->\n\n# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Contributor Covenant Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, religion, or sexual identity\nand orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Contributor Covenant Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the\n  overall community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or\n  advances of any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email\n  address, without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Contributor Covenant Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Contributor Covenant Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official e-mail address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Contributor Covenant Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement by sending a\nmessage to [Rosa Richter on keybase.io](https://keybase.io/cantido).\nAll complaints will be reviewed and investigated promptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Contributor Covenant Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Contributor Covenant Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Contributor Covenant Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series\nof actions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or\npermanent ban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Contributor Covenant Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Contributor Covenant Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior,  harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within\nthe community.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Contributor Covenant Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.0, available at\nhttps://www.contributor-covenant.org/version/2/0/code_of_conduct.html.\n\nCommunity Impact Guidelines were inspired by [Mozilla's code of conduct\nenforcement ladder](https://github.com/mozilla/diversity).\n\n[homepage]: https://www.contributor-covenant.org\n\nFor answers to common questions about this code of conduct, see the FAQ at\nhttps://www.contributor-covenant.org/faq. Translations are available at\nhttps://www.contributor-covenant.org/translations.","ref":"code_of_conduct.html#attribution"},{"type":"extras","title":"Getting Started","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Getting Started\n\nLet's get a Liberator endpoint up and running as fast as possible.\n\nFirst, you'll probably want to fire up a Phoenix application.\nCheck out [Phoenix's Installation Guide](https://hexdocs.pm/phoenix/installation.html)\nand [their Up and Running Guide](https://hexdocs.pm/phoenix/up_and_running.html)\nand install it if you don't have it already,\nthen create a new project with `phx.new`.\n\nSince Liberator was built to help create APIs,\nwe will disable HTML generation.\nWe'll keep Ecto, though.\n\n```sh\n$ mix phx.new hello --no-html --no-webpack\n```\n\n> We'll skip the details of setting up an Ecto repo,\n> because it's not necessary to have one running to use Liberator.\n> However, your data probably lives in a repo,\n> so this example will contain some references to one.\n\nAdd `liberator` to your list of dependencies in `mix.exs`\n\n```elixir\ndef deps do\n  [\n    {:liberator, \"~> 1.3.0\"}\n  ]\nend\n```\n\nThen we'll use `mix phx.gen.json` to create a basic schema, route, and context.\nIt will create a controller as well, but we'll be replacing that with a Liberator Resource.\n\n```sh\n$ mix phx.gen.json Blog Post posts title:string content:string\n```\n\nMigrate the database as the command feedback requests, if you've got your database handy.\nHowever, don't copy the statement they're asking you to put in `lib/hello_web/router.ex`.\nWe're going to use our own.\nIn `lib/hello_web/router.ex`, forward any `/posts` requests to the resource we're about to define.\n\n```elixir\ndefmodule HelloWeb.Router do\n  use HelloWeb, :router\n\n  forward \"/posts\", HelloWeb.PostsResource\nend\n```\n\nNow we must actually create our first Liberator resource.\nGo ahead and delete the file at `lib/controllers/posts_controller.ex` that Phoenix generated for you,\nand create a file at `lib/controllers/posts_resource.ex` with the following content.\n\n```elixir\ndefmodule HelloWeb.PostsResource do\n  use Liberator.Resource\nend\n```\n\nRun the project with `mix phx.server` and check out `http://localhost:4000/posts`.\nWith Liberator's sensible defaults, it should return a status of 200 and content of `OK`.\n\n```sh\n$ curl --location --request GET 'http://localhost:4000/posts/'\nHTTP/1.1 200 OK\nallow: GET, HEAD\ncache-control: max-age=0, private, must-revalidate\ncontent-encoding: identity\ncontent-length: 2\ncontent-type: text/plain\ndate: Thu, 15 Oct 2020 22:54:57 GMT\nserver: Cowboy\nx-request-id: Fj5MYV-9JH1y19wAACQl\n\n\nOK\n```\n\n\nLet's spice that up a little more and get some dynamic content in here.\n\n```elixir\ndefmodule HelloWeb.PostsResource do\n  use Liberator.Resource\n\n  @impl true\n  def handle_ok(_conn) do\n    DateTime.utc_now() |> DateTime.to_iso8601()\n  end\nend\n```\n\n\n```sh\n$ curl --location --request GET 'http://localhost:4000/posts/'\nHTTP/1.1 200 OK\nallow: GET, HEAD\ncache-control: max-age=0, private, must-revalidate\ncontent-encoding: identity\ncontent-length: 27\ncontent-type: text/plain\ndate: Thu, 15 Oct 2020 22:53:42 GMT\nserver: Cowboy\nx-request-id: Fj5MT91NPreSB7QAABqF\n\n\n2020-10-15T22:53:30.668000Z\n```\n\nThat's barely better than regular Phoenix controllers, so let's go further.\nSince we're trying to write an API, why don't we grab our Repo and try to get some data out in the world?\n\n```elixir\ndefmodule HelloWeb.PostsResource do\n  use Liberator.Resource\n\n  @impl true\n  def handle_ok(conn) do\n    id = List.last(conn.path_info)\n    post = Hello.Blog.get_post!(id)\n\n    post.title <> \"\\n\\n\" <> post.content\n  end\nend\n```\n\nNow if you try to hit `http://localhost:4000/posts/5`, we get a Phoenix exception.\nNothing is in the database, of course.\nWe must check to see if the given ID exists in the database,\nand here's where Liberator's features start to show.\n\nImplement the `c:Liberator.Resource.exists?/1` callback in your resource.\nHere we can check for the resource.\nIn fact, let's just grab the entire thing and stick in back in the `conn`.\nThe `exists?/1` callback expects you to return a boolean, a `Map`, or the `conn`.\nIf you return a `Map`, Liberator will merge that into `conn.assigns`.\n\n```elixir\ndefmodule HelloWeb.PostsResource do\n  use Liberator.Resource\n\n  @impl true\n  def exists?(conn) do\n    id = List.last(conn.path_info)\n    try do\n      Hello.Blog.get_post!(id)\n    rescue\n      Ecto.NoResultsError -> false\n      ArgumentError -> false\n    else\n      post -> %{post: post}\n    end\n  end\n\n  @impl true\n  def handle_ok(conn) do\n    post = conn.assigns[:post]\n    post.title <> \" \" <> post.content\n  end\nend\n```\n\n```sh\n$ curl --location --request GET 'http://localhost:4000/posts/1555'\nHTTP/1.1 404 Not Found\nallow: GET, HEAD\ncache-control: max-age=0, private, must-revalidate\ncontent-encoding: identity\ncontent-length: 9\ncontent-type: text/plain\ndate: Thu, 15 Oct 2020 22:52:25 GMT\nserver: Cowboy\nx-request-id: Fj5MPb9fgMWho78AABkF\n\n\nNot Found\n```\n\nNow we get a nice \"Not Found\" response!\nSo, last, we want to be able to POST something.\nThat introduces us to the concept of actions,\nwhich are just another kind of overridable function.\nThere are four actions in Liberator:\n\n- `delete!`\n- `patch!`\n- `post!`\n- `put!`\n\nAs you could guess, these actions will be called based on the request's HTTP method.\nFor now, we'll set up `post!`.\nAdd `c:Liberator.Resource.post!/1` to insert the params.\nWe also want to make sure that we allow POST requests,\nso define `c:Liberator.Resource.allowed_methods/1` too.\n\nWe could parse the request body here if we wanted, but Plug provides the `Plug.Parsers` plug,\nand it's already set up in this beginner project for accepting JSON.\nThe params are available in `conn.params`, so we can pass those into our context module directly,\nand let the changeset handle validation.\n\n```elixir\ndefmodule HelloWeb.PostsResource do\n  use Liberator.Resource\n\n  @impl true\n  def allowed_methods(_), do: [\"POST\", \"GET\"]\n\n  @impl true\n  def exists?(conn) do\n    id = List.last(conn.path_info)\n    case Hello.Repo.get(Hello.Blog.Post, id) do\n      nil -> false\n      post -> %{post: post}\n    end\n  end\n\n  @impl true\n  def post!(conn) do\n    {:ok, _post} = Hello.Blog.create_post(conn.params)\n  end\n\n  @impl true\n  def handle_ok(conn) do\n    post = conn.assigns[:post]\n    post.title <> \" \" <> post.content\n  end\nend\n```\n\nThat's all!\nLet's POST to our new endpoint now.\n\n```sh\n$ curl --location --request POST 'http://localhost:4000/posts' \\\n> --header 'Content-Type: application/json' \\\n> --data-raw '{\n>     \"title\": \"My first post!\",\n>    \"content\": \"This is so fun!\"\n> }'\nHTTP/1.1 201 Created\nallow: POST, GET\ncache-control: max-age=0, private, must-revalidate\ncontent-encoding: identity\ncontent-length: 7\ncontent-type: text/plain\ndate: Thu, 15 Oct 2020 22:42:07 GMT\nserver: Cowboy\nx-request-id: Fj5L-UYbiai2oD4AADaB\n\n\nCreated\n```\n\nFinally, point your browser at http://localhost:4000/posts/1.\n\n```sh\ncurl --location --request GET 'http://localhost:4000/posts/1'\nHTTP/1.1 200 OK\nallow: POST, GET\ncache-control: max-age=0, private, must-revalidate\ncontent-encoding: identity\ncontent-length: 30\ncontent-type: text/plain\ndate: Thu, 15 Oct 2020 22:47:30 GMT\nserver: Cowboy\nx-request-id: Fj5LreF6rvXe6igAABoD\n\n\nMy first post! This is so fun!\n```\n\n\nYou've done it!\nYou've just built JSON endpoint with a fair amount of built-in smarts.\nThere's so much more to customize, but now you've got a base to work from.\nCheck out the documentation for [`Liberator.Resource`](https://hexdocs.pm/liberator/Liberator.Resource.html)\nto see all the stuff you can do!","ref":"getting_started.html"},{"type":"extras","title":"Decision Tree","doc":"<!--\nSPDX-FileCopyrightText: 2024 Rosa Richter\n\nSPDX-License-Identifier: CC-BY-SA-4.0\n-->\n\n# Decision Tree\n\nThere are a lot of decision to be made before Liberator sends a response.\nThis is the current state of Liberator's decision tree, with handlers at the bottom.\n\n![decision tree](default_decision_tree.svg)","ref":"decision_tree.html"},{"type":"extras","title":"Creating your own chart - Decision Tree","doc":"You can create Graphviz source code for your own decision tree using the `liberator.chart` mix task.\n\n```sh\nmix liberator.chart -o mychart.dot MyApp.MyResource\n```\n\nYou will have to install [Graphviz](https://graphviz.org/)\nor use one of its many language bindings to generate the actual image.\nUnfortunately, there isn't a binding for Elixir or Erlang.","ref":"decision_tree.html#creating-your-own-chart"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}